@@ -1,8 +1,10 @@
 """Kamereon API."""
 import logging
+from json import dumps as json_dumps
 from typing import Any
 from typing import cast
 from typing import Dict
+from typing import List
 from typing import Optional
 
 import aiohttp
@@ -15,27 +17,27 @@ from . import schemas
 _LOGGER = logging.getLogger(__name__)
 
 
-DATA_ENDPOINTS = {
+DATA_ENDPOINTS: Dict[str, Any] = {
     "battery-status": {"version": 2},
     "charge-history": {"version": 1},
-    "charge-mode": {"version": 1},
+    "charge-mode": {"version": 1, "requires-contracts": "ZEINTER"},
     "charges": {"version": 1},
-    "charging-settings": {"version": 1},
+    "charging-settings": {"version": 1, "requires-contracts": "ZEINTER"},
     "cockpit": {"version": 2},
-    "location": {"version": 1},
-    "hvac-history": {"version": 1},
-    "hvac-sessions": {"version": 1},
-    "hvac-status": {"version": 1},
+    "hvac-history": {"version": 1, "requires-contracts": "ZEINTER"},
+    "hvac-sessions": {"version": 1, "requires-contracts": "ZEINTER"},
+    "hvac-status": {"version": 1, "requires-contracts": "ZEINTER"},
     "hvac-settings": {"version": 1},
+    "location": {"version": 1},
     "lock-status": {"version": 1},
     "notification-settings": {"version": 1},
 }
-ACTION_ENDPOINTS = {
+ACTION_ENDPOINTS: Dict[str, Any] = {
     "charge-mode": {"version": 1, "type": "ChargeMode"},
     "charge-schedule": {"version": 2, "type": "ChargeSchedule"},
     "charging-start": {"version": 1, "type": "ChargingStart"},
-    "hvac-start": {"version": 1, "type": "HvacStart"},
     "hvac-schedule": {"version": 2, "type": "HvacSchedule"},
+    "hvac-start": {"version": 1, "type": "HvacStart"},
 }
 
 
@@ -60,6 +62,36 @@ def get_car_adapter_url(root_url: str, account_id: str, version: int, vin: str)
     return f"{account_url}/kamereon/kca/car-adapter/v{version}/cars/{vin}"
 
 
+def get_contracts_url(root_url: str, account_id: str, vin: str) -> str:
+    """Get the url to the car contracts."""
+    account_url = get_account_url(root_url, account_id)
+    return f"{account_url}/vehicles/{vin}/contracts"
+
+
+def get_required_contracts(endpoint: str) -> str:
+    """Get the required contracts for the specified endpoint."""
+    endpoints = ACTION_ENDPOINTS if endpoint.startswith("action") else DATA_ENDPOINTS
+    return str(endpoints.get(endpoint, {}).get("requires-contracts", ""))
+
+
+def has_required_contracts(
+    contracts: List[models.KamereonVehicleContract], endpoint: str
+) -> bool:
+    """Check if vehicle has contract for endpoint."""
+    required_contracts = get_required_contracts(endpoint)
+    if not required_contracts:
+        return True
+
+    for required_contract in required_contracts.split(","):
+        if required_contract and not any(
+            contract.code == required_contract and contract.status == "ACTIVE"
+            for contract in contracts
+        ):
+            return False
+
+    return True
+
+
 async def request(
     websession: aiohttp.ClientSession,
     method: str,
@@ -69,6 +101,8 @@ async def request(
     params: Dict[str, str],
     json: Optional[Dict[str, Any]] = None,
     schema: Optional[Schema] = None,
+    *,
+    wrap_array_in: Optional[str] = None,
 ) -> models.KamereonResponse:
     """Process Kamereon HTTP request."""
     schema = schema or schemas.KamereonResponseSchema
@@ -85,14 +119,25 @@ async def request(
         json=json,
     ) as http_response:
         response_text = await http_response.text()
+        if json:
+            _LOGGER.debug(
+                "Send Kamereon %s request to %s with body: %s",
+                method,
+                http_response.url,
+                json_dumps(json),
+            )
         _LOGGER.debug(
-            "Received Kamereon response %s on %s %s: %s (with body data %s)",
+            "Received Kamereon response %s on %s to %s: %s",
             http_response.status,
             method,
             http_response.url,
             response_text,
-            json,
         )
+
+        # Some endpoints return arrays instead of objects.
+        # These need to be wrapped in an object.
+        if response_text.startswith("[") and wrap_array_in:
+            response_text = f'{{"{wrap_array_in}": {response_text}}}'
         kamereon_response: models.KamereonResponse = schema.loads(response_text)
         # Check for Kamereon error
         kamereon_response.raise_for_error_code()
@@ -127,6 +172,42 @@ async def get_person(
     )
 
 
+async def get_vehicle_contracts(
+    websession: aiohttp.ClientSession,
+    root_url: str,
+    api_key: str,
+    gigya_jwt: str,
+    country: str,
+    locale: str,
+    account_id: str,
+    vin: str,
+) -> models.KamereonVehicleContractsResponse:
+    """GET to /accounts/{accountId}/vehicles/{vin}/contracts."""
+    url = get_contracts_url(root_url, account_id, vin)
+    params = {
+        "country": country,
+        "locale": locale,
+        "brand": "RENAULT",
+        "connectedServicesContracts": "true",
+        "warranty": "true",
+        "warrantyMaintenanceContracts": "true",
+    }
+
+    return cast(
+        models.KamereonVehicleContractsResponse,
+        await request(
+            websession,
+            "GET",
+            url,
+            api_key,
+            gigya_jwt,
+            params=params,
+            schema=schemas.KamereonVehicleContractsResponseSchema,
+            wrap_array_in="contractList",
+        ),
+    )
+
+
 async def get_account_vehicles(
     websession: aiohttp.ClientSession,
     root_url: str,
@@ -152,6 +233,32 @@ async def get_account_vehicles(
     )
 
 
+async def get_vehicle_details(
+    websession: aiohttp.ClientSession,
+    root_url: str,
+    api_key: str,
+    gigya_jwt: str,
+    country: str,
+    account_id: str,
+    vin: str,
+) -> models.KamereonVehicleDetailsResponse:
+    """GET to /accounts/{account_id}/vehicles/{vin}/details."""
+    url = f"{get_account_url(root_url, account_id)}/vehicles/{vin}/details"
+    params = {"country": country}
+    return cast(
+        models.KamereonVehicleDetailsResponse,
+        await request(
+            websession,
+            "GET",
+            url,
+            api_key,
+            gigya_jwt,
+            params=params,
+            schema=schemas.KamereonVehicleDetailsResponseSchema,
+        ),
+    )
+
+
 def _get_endpoint_version(endpoint_details: Dict[str, Any]) -> int:
     return int(endpoint_details["version"])
 
