@@ -41,6 +41,23 @@ COMMON_ERRRORS: List[Dict[str, Any]] = [
     },
 ]
 
+VEHICLE_SPECIFICATIONS: Dict[str, Dict[str, Any]] = {
+    "X071VE": {  # TWINGO III
+        "support-endpoint-hvac-status": False,
+    },
+    "X101VE": {  # ZOE phase 1
+        "reports-in-watts": True,
+        "support-endpoint-location": False,
+    },
+    "X102VE": {  # ZOE phase 2
+        "support-endpoint-hvac-status": False,
+        "warns-on-method-set_ac_stop": "Action `cancel` on endpoint `hvac-start` may not be supported on this model.",  # noqa
+    },
+    "XJB1SU": {  # CAPTUR II
+        "support-endpoint-hvac-status": False,
+    },
+}
+
 
 @dataclass
 class KamereonResponseError(BaseModel):
@@ -110,11 +127,11 @@ class KamereonPersonAccount(BaseModel):
 class KamereonPersonResponse(KamereonResponse):
     """Kamereon response to GET on /persons/{gigya_person_id}."""
 
-    accounts: List[KamereonPersonAccount]
+    accounts: Optional[List[KamereonPersonAccount]]
 
 
 @dataclass
-class KamereonVehiclesDetailsGroup(BaseModel):
+class KamereonVehicleDetailsGroup(BaseModel):
     """Kamereon vehicle details group data."""
 
     code: Optional[str]
@@ -123,14 +140,15 @@ class KamereonVehiclesDetailsGroup(BaseModel):
 
 
 @dataclass
-class KamereonVehiclesDetails(BaseModel):
+class KamereonVehicleDetails(BaseModel):
     """Kamereon vehicle details."""
 
     vin: Optional[str]
     registrationNumber: Optional[str]  # noqa: N815
-    brand: Optional[KamereonVehiclesDetailsGroup]
-    model: Optional[KamereonVehiclesDetailsGroup]
-    energy: Optional[KamereonVehiclesDetailsGroup]
+    radioCode: Optional[str]  # noqa: N815
+    brand: Optional[KamereonVehicleDetailsGroup]
+    model: Optional[KamereonVehicleDetailsGroup]
+    energy: Optional[KamereonVehicleDetailsGroup]
 
     def get_energy_code(self) -> Optional[str]:
         """Return vehicle energy code."""
@@ -168,11 +186,30 @@ class KamereonVehiclesDetails(BaseModel):
 
     def reports_charging_power_in_watts(self) -> bool:
         """Return True if model reports chargingInstantaneousPower in watts."""
-        if self.get_model_code() in [
-            "X101VE",  # ZOE 40
-        ]:
-            return True
-        return False
+        # Default to False for unknown vehicles
+        if self.model and self.model.code:
+            return VEHICLE_SPECIFICATIONS.get(self.model.code, {}).get(
+                "reports-in-watts", False
+            )
+        return False  # pragma: no cover
+
+    def supports_endpoint(self, endpoint: str) -> bool:
+        """Return True if model supports specified endpoint."""
+        # Default to True for unknown vehicles
+        if self.model and self.model.code:
+            return VEHICLE_SPECIFICATIONS.get(self.model.code, {}).get(
+                f"support-endpoint-{endpoint}", True
+            )
+        return True  # pragma: no cover
+
+    def warns_on_method(self, method: str) -> Optional[str]:
+        """Return warning message if model trigger a warning on the method call."""
+        # Default to None for unknown vehicles
+        if self.model and self.model.code:
+            return VEHICLE_SPECIFICATIONS.get(self.model.code, {}).get(
+                f"warns-on-method-{method}", None
+            )
+        return None  # pragma: no cover
 
 
 @dataclass
@@ -180,7 +217,7 @@ class KamereonVehiclesLink(BaseModel):
     """Kamereon vehicles link data."""
 
     vin: Optional[str]
-    vehicleDetails: Optional[KamereonVehiclesDetails]  # noqa: N815
+    vehicleDetails: Optional[KamereonVehicleDetails]  # noqa: N815
 
 
 @dataclass
@@ -189,7 +226,12 @@ class KamereonVehiclesResponse(KamereonResponse):
 
     accountId: Optional[str]  # noqa: N815
     country: Optional[str]
-    vehicleLinks: List[KamereonVehiclesLink]  # noqa: N815
+    vehicleLinks: Optional[List[KamereonVehiclesLink]]  # noqa: N815
+
+
+@dataclass
+class KamereonVehicleDetailsResponse(KamereonResponse, KamereonVehicleDetails):
+    """Kamereon response to GET on /accounts/{account_id}/vehicles/{vin}/details."""
 
 
 @dataclass
@@ -197,6 +239,29 @@ class KamereonVehicleDataAttributes(BaseModel):
     """Kamereon vehicle data attributes."""
 
 
+@dataclass
+class KamereonVehicleContract(BaseModel):
+    """Kamereon vehicle contract."""
+
+    type: Optional[str]
+    contractId: Optional[str]  # noqa: N815
+    code: Optional[str]
+    group: Optional[str]
+    durationMonths: Optional[int]  # noqa: N815
+    startDate: Optional[str]  # noqa: N815
+    endDate: Optional[str]  # noqa: N815
+    status: Optional[str]
+    statusLabel: Optional[str]  # noqa: N815
+    description: Optional[str]
+
+
+@dataclass
+class KamereonVehicleContractsResponse(KamereonResponse):
+    """Kamereon response to GET on /accounts/{accountId}/vehicles/{vin}/contracts."""
+
+    contractList: Optional[List[KamereonVehicleContract]]  # noqa: N815
+
+
 @dataclass
 class KamereonVehicleData(BaseModel):
     """Kamereon vehicle data."""
@@ -319,15 +384,10 @@ class ChargeDaySchedule(BaseModel):
         }
 
     def get_end_time(self) -> Optional[str]:
-        """Create dict for json."""
+        """Get end time."""
         if self.startTime is None:  # pragma: no cover
             return None
-        start_hours = int(self.startTime[1:3])
-        start_minutes = int(self.startTime[4:6])
-        total_minutes = start_hours * 60 + start_minutes + (self.duration or 0)
-        end_hours, end_minutes = divmod(total_minutes, 60)
-        end_hours = end_hours % 24
-        return f"T{end_hours:02g}:{end_minutes:02g}Z"
+        return helpers.get_end_time(self.startTime, self.duration)
 
 
 @dataclass
